#define _XOPEN_SOURCE 1
#define _XOPEN_SOURCE_EXTENDED 1
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

static char const prog[]="supervise-wrapper";

extern char const* const* environ;

static struct stat statbuf;
stralloc sa={ 0 };

void die_alloc(void) {
  strerr_die5sys(111, prog, fatal, err_unable, err_alloc, err_);
}

int main(int argc, char** argv) {
  int const flags=fcntl(fileno_stdin, F_GETFL);
  int const islog=(fstat(fileno_stdin, &statbuf)==0 &&
                   S_ISFIFO(statbuf.st_mode));

  if (argc<3)
    strerr_die4x(100, prog, usage, prog, " supervise dir");

  if (flags==-1)
    strerr_die5sys(100, prog, fatal, err_unable, err_getflags, err_);

  if (stralloc_copys(&sa, argv[argc-1])==0 ||
      stralloc_append(&sa, "/")==0 ||
      stralloc_cats(&sa, SVCLEAN)==0 ||
      stralloc_cats(&sa, SVCLEAN_SHUTDOWN)==0 ||
      stralloc_0(&sa)==0)
    die_alloc();
  if (stat(sa.s, &statbuf)==0) {
    if (!islog) _exit(0);
    {
      iopause_fd iop;
      struct taia deadline, stamp;
      iop.fd=fileno_stdin;
      iop.events=IOPAUSE_READ;
      taia_now(&deadline);
      taia_now(&stamp);
      iopause(&iop, 1, &deadline, &stamp);
      if ((iop.revents&IOPAUSE_READ)==0)
        _exit(0);
    }
    if ((flags&nonblock)==0)
      if (fcntl(fileno_stdin, F_SETFL, flags|nonblock)==-1)
        strerr_die5sys(100, prog, fatal, err_unable, err_setflags, err_);
  } else
    if (islog)
      if ((flags&nonblock)!=0)
        if (fcntl(fileno_stdin, F_SETFL, flags&~nonblock)==-1)
          strerr_die5sys(100, prog, fatal, err_unable, err_setflags, err_);

  if (stralloc_copys(&sa, argv[argc-1])==0 ||
      stralloc_append(&sa, "/")==0 ||
      stralloc_cats(&sa, SVCLEAN)==0 ||
      stralloc_cats(&sa, SVCLEAN_KEEP_5)==0 ||
      stralloc_0(&sa)==0)
    die_alloc();
  if (stat(sa.s, &statbuf)!=0) close(fileno_piperd);
  sa.s[sa.len-2]='0'+fileno_newerr;
  if (stat(sa.s, &statbuf)!=0) close(fileno_newerr);
  sa.s[sa.len-2]='0'+fileno_newout;
  if (stat(sa.s, &statbuf)!=0) close(fileno_newout);

  if (stralloc_copys(&sa, argv[argc-1])==0 ||
      stralloc_append(&sa, "/")==0 ||
      stralloc_cats(&sa, SVCLEAN)==0 ||
      stralloc_cats(&sa, SVCLEAN_REDIRECT_2_1)==0 ||
      stralloc_0(&sa)==0)
    die_alloc();
  if (stat(sa.s, &statbuf)==0)
    if (dup2(fileno_stdout, fileno_stderr)!=fileno_stderr)
      strerr_die5sys(100, prog, fatal, err_unable, err_dup, err_);

  ++argv;
  pathexec_run(*argv, (char const**)argv, environ);
  strerr_die6sys(errstat(errno), prog, fatal, err_unable, err_exec, argv[0],
                 err_);
}
