#define _XOPEN_SOURCE 1
#define _XOPEN_SOURCE_EXTENDED 1
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

static char const prog[]="svclean-svscan";

static char const var_suppath[]="$SUPERVISE_PATH";

int main(int argc, char** argv) {
  int logpipe[2];

  if (argc<2)
    strerr_die4x(100, prog, usage, prog, " svscan [/service]");

  close(fileno_piperd);

  if ((dup2(fileno_stdout, fileno_newout)!=fileno_newout && errno!=EBADF) ||
      (dup2(fileno_stderr, fileno_newerr)!=fileno_newerr && errno!=EBADF))
    strerr_die5sys(100, prog, fatal, err_unable, err_dup, err_);
  buffer_2->fd=fileno_newerr;

  close(fileno_stdout);
  close(fileno_stderr);

  if (pipe(logpipe)!=0)
    strerr_die6sys(errstat(errno), prog, fatal, err_unable, err_create,
                   err_pipe, err_);

  {
    int const flags=fcntl(logpipe[pipe_wr], F_GETFL);
    if (flags==-1)
      strerr_die5sys(100, prog, fatal, err_unable, err_getflags, err_);
    if (fcntl(logpipe[pipe_wr], F_SETFL, flags|nonblock)==-1)
      strerr_die5sys(100, prog, fatal, err_unable, err_setflags, err_);
  }

  if (logpipe[pipe_wr]==fileno_piperd) {
    int wr0, wr1;
    if (logpipe[pipe_rd]==fileno_stdout) {
      wr0=fileno_stderr;
      wr1=fileno_stdout;
    } else {
      wr0=fileno_stdout;
      wr1=fileno_stderr;
    }
    if (dup2(logpipe[pipe_wr], wr0)!=wr0) goto oops;
    if (dup2(logpipe[pipe_rd], fileno_piperd)!=fileno_piperd) goto oops;
    close(logpipe[pipe_rd]);
    if (dup2(wr0, wr1)!=wr1) goto oops;
  } else {
    if (logpipe[pipe_rd]!=fileno_piperd) {
      if (dup2(logpipe[pipe_rd], fileno_piperd)!=fileno_piperd) goto oops;
      close(logpipe[pipe_rd]);
    }
    if (logpipe[pipe_wr]==fileno_stdout) {
      if (dup2(logpipe[pipe_wr], fileno_stderr)!=fileno_stderr) goto oops;
    } else {
      if (dup2(logpipe[pipe_wr], fileno_stdout)!=fileno_stdout) goto oops;
      if (logpipe[pipe_wr]!=fileno_stderr) {
        close(logpipe[pipe_wr]);
        if (dup2(fileno_stdout, fileno_stderr)!=fileno_stderr) goto oops;
      }
    }
  }
  if (0)
    oops: strerr_die5sys(100, prog, fatal, err_unable, err_dup, err_);

  if (pathexec_env(var_suppath+1, env_get(var_path+1))==0)
    strerr_die6sys(111, prog, fatal, err_unable, err_setvar, var_suppath,
                   err_);
  if (pathexec_env(var_path+1, PACKAGE_HOME "/libexec")==0)
    strerr_die6sys(111, prog, fatal, err_unable, err_setvar, var_path, err_);

  ++argv;
  pathexec((char const**)argv);
  strerr_die6sys(errstat(errno), prog, fatal, err_unable, err_exec, argv[0],
                 err_);
}
